/*
   Carrier monitor. v1.02
   Copyright (c) 1991, Max Mikheenkov and Denis Julitov

   This hook allows to preserve Int 14h carrier detect status during
   temporary carrier losses. It's useful for using with mailers that
   terminate mail sessions on any carrier loss and ignore carrier
   re-establishing.

   Usage:

       cm <port> <wait_CD_ticks> <fake_CD_ticks>

   Where:
       <port>          - port number ( starts from 0) 
       <wait_CD_ticks> - valid carrier loss gap length in CLOCK_TICKs
                         ( usually 1/18 sec.)
       <fake_CD_ticks> - CONNECT establishing time in CLOCK_TICKs
			 
   Hints:
       You need to run CM when fossil is loaded. Also you should tune
       this program and your modem accordingly. Hope, you will have
       no problems to choose s10 and <wait_CD_ticks>. It depends of
       your line quality and modem sensitiness. Do not forget to set s9
       correctly too ( look thro your modem guide for description of this
       registers). The <fake_CD_ticks> parameter means an inerval when CM
       still is transparent during connection start to establish first
       time. It was added to the program to suppress supporting of fake
       carrier spikes that happen sometimes. The best way is to rise
       this parameter step by step till you have no the "carrier detect",
       "carrier lost" sequence during a modem initialization. If you need
       more hints -- just look following code :)

   Example:

       modem settings: ATS9=4S10=30
       cm 0 54 18

  Compiling ( Microsoft C ver. 6.0):

  	cl /Ox /Lr /F 100 cm.c

  A pair official words:
       This program is freeware. You can feel free to use and distribute
       CM without fee and code modifications. Nothing is guaranted.

  Please, send your comments to:

  Fido  : Max Mikheenkov, 2:5020/9
  e-mail: max@imech.msk.su
*/

#include <conio.h>
#include <stdlib.h>
#include <dos.h>

#pragma check_stack( off )
#pragma check_pointer( off )

typedef void		VOID;
typedef void far *	PVOID;
typedef unsigned char	UCHAR;
typedef unsigned short	USHORT;
typedef unsigned short	BOOL;
typedef unsigned long	ULONG;
typedef ULONG far *	PULONG;

#define IFN		interrupt far	
#define PIFN		interrupt far *	

/* Hook mode definition */
				/* Hook ... */
#define	NOC		0	/* waits for carrier */
#define	CD		1	/* waits for carrier loss :-) */
#define CTEST		2	/* makes sure connect established */
#define CLGAP		3	/* waits for connect will re-establish */
#define NTD		4	/* Nothing to do. DTR was dropped */

#define	FOSSIL		0x14
#define GET_STATUS	0x03
#define SET_DTR		0x06
#define TICK_COUNTER	0x0000046cL
#define CD_MASK		0x0080
#define DCD_MASK	0x0008
#define DROP_DTR_MASK   0x00

VOID ( PIFN Fossil)( void );
VOID IFN FossilHook( USHORT _es, USHORT _ds, USHORT _di,
		     USHORT _si, USHORT _bp, USHORT _sp,
		     USHORT _bx, USHORT _dx, USHORT _cx,
		     USHORT _ax, USHORT _ip, USHORT _cs,
		     USHORT _flags);

static char huge	*tsrstack;
static char huge	*tsrbottom;

static USHORT		usPortNumber;
static ULONG		ulCValidPause;
static ULONG		ulCTestTime;

void main( int argc, char **argv ) {

    unsigned	tsrsize;
 
    _asm mov	WORD PTR tsrstack[0], sp
    _asm mov	WORD PTR tsrstack[2], ss
    FP_SEG( tsrbottom ) = _psp;
    FP_OFF( tsrbottom ) = 0;
    tsrsize = ((tsrstack - tsrbottom) >> 4) + 1;

    cputs( "\r\nCM. Carrier monitor v1.02\r\n"
           "Copyright (c) 1991, Max Mikheenkov and Denis Julitov\r\n" );

    if( argc != 4 ) {		/* Yeah, it isn't foolproof :-) */

        cputs( "\r\nUsage: cm <port> <wait_CD_ticks> "
	       "<fake_CD_ticks>\r\n\r\n");
	exit(1);
    }

    usPortNumber = atoi( argv[1]);
    ulCValidPause = atol( argv[2]);
    ulCTestTime = atol( argv[3]);
    Fossil = _dos_getvect( FOSSIL);
    _dos_setvect( FOSSIL, FossilHook);

   _dos_freemem( _psp );
   _dos_keep( 0, tsrsize );
}

VOID IFN FossilHook( USHORT _es, USHORT _ds, USHORT _di,
		     USHORT _si, USHORT _bp, USHORT _sp,
		     USHORT _bx, USHORT _dx, USHORT _cx,
		     USHORT _ax, USHORT _ip, USHORT _cs,
		     USHORT _flags) {

    static USHORT 		usMode = (USHORT) NOC;
    static ULONG		ulEndOfTime = 0L;
    static volatile PULONG	pulTickCounter = (PVOID) TICK_COUNTER;
    BOOL			fCarrier;

    /* "Drop DTR" */

    if( (UCHAR) (_ax >> 8) == (UCHAR) SET_DTR && _dx == usPortNumber && 
        (UCHAR) _ax == (UCHAR) DROP_DTR_MASK) {

	usMode = NTD;
    }

    /* "Get modem status" hook follows */

    if( (UCHAR) (_ax >> 8) == (UCHAR) GET_STATUS && _dx == usPortNumber) {

	_asm  mov	dx, _dx
	_asm  mov	ax, _ax

	( *Fossil)();

	_asm  mov	_ax, ax

	fCarrier = _ax & CD_MASK;

	switch( usMode) {

	case NOC:

	    if( fCarrier) {

	        usMode = CTEST;
	        ulEndOfTime = *pulTickCounter + ulCTestTime;
	    }
	    break;

	case CD:

	    if( !fCarrier) {

		usMode = CLGAP;
	        ulEndOfTime = *pulTickCounter + ulCValidPause;
		_ax |= CD_MASK;
		_ax &= ~DCD_MASK;
	    }
	    break;

	case CTEST:

	    if( *pulTickCounter < ulEndOfTime && !fCarrier) {

	        usMode = NOC;

	    } else {

	        usMode = CD;
	    }
	    break;

	case CLGAP:

	    if( fCarrier) {

		usMode = CD;
		_ax &= ~DCD_MASK;

	    } else {

		if( *pulTickCounter < ulEndOfTime) {

		    _ax |= CD_MASK;

		} else {	    

		    usMode = NOC;
		    _ax |= DCD_MASK;
		}
	    }
	    break;

	case NTD:

	    if( !fCarrier) {

	        usMode = NOC;
	    }
	}
    } else {
    
	_chain_intr( Fossil);
    }
}
